#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QPropertyAnimation>
#include <QTimer>
#include <QPixmap>
#include <QBitmap>
#include <QGraphicsBlurEffect>
#include <QFont>
#include <QCloseEvent>
#include <QMutex>
#include <QMutexLocker>
#include <QScrollArea>
#include <QWinTaskbarButton>

#include "statuswidget.h"
#include "aboutwidget.h"
#include "helpwidget.h"
#include "legacyapplication.h"
#include "widgets/newslistwidget.h"

namespace Ui {
class MainWindow;
}

class MenuEntry;
class LotroDatManager;
class PatchDownloader;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    NewsListWidget* getNewsWidget();
    QScrollArea* getNewsWidgetOverlay();

public slots:
    void updateFontSizes();

signals:
    void messageDialogFinished(QObject* emitter, int result);

protected:
    virtual void closeEvent(QCloseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void resizeEvent(QResizeEvent *event) override;
    bool eventFilter(QObject *watched, QEvent *event) override;

private slots:
    void randomChangeBackground();

    void on_menuentry_1_clicked();

    void on_menuentry_2_clicked();

    void on_menuentry_4_clicked();

    void onHoverMenuentry(MenuEntry* hovered_entry);

    void on_closeButton_clicked();

    void on_minimizeButton_clicked();

    void setupWindowBackgroundAndMask(QPixmap background);

    void onPatchDownloaderStarted();

    void onPatchDownloaderFinished();

    void onPatchDownloaderProgressChanged(Downloader::Status status);

    void onPatchInstallerStarted();

    void onPatchInstallerFinished();

    void onPatchInstallerProgressChanged(PatchInstaller::Status status);

    void onPatchInstallerVideoProgressChanged(int finished_videos, int total_videos, Downloader::Status status);

private:
    void makeConnections();

    void setupMenuHoverWidget();

    void moveMenuHoverWidget(MenuEntry* target);

    void checkMenuIsHovered();

    void hideAllContentWidgets();

    bool buttonsMouseMoveEventFilter(QObject *obj, QEvent *event);

    void setEventFilterRecursive(QObject* widget);

private:
    bool show_warning_on_close = false;

    Ui::MainWindow *ui = nullptr;

    QWinTaskbarButton* _windows_taskbar_button = nullptr;

    MenuEntry* current_active_entry_ = nullptr;
    MenuEntry* current_hovered_entry_ = nullptr;

    StatusWidget *status_widget_ = nullptr;
    HelpWidget *help_widget_ = nullptr;
    AboutWidget *about_widget_ = nullptr;

    QWidget* menu_hover_widget_ = nullptr;
    QPropertyAnimation* menu_hover_widget_animation_ = nullptr;
    QTimer menu_hover_checker_timer_;

    QPoint drag_position_;

    int current_bg_id_;
    QPixmap current_bg_;
    QBitmap current_mask_;
    QTimer background_update_timer_;

    const int MAX_PIXMAP_ID = 21;
};


#endif // MAINWINDOW_H
