#include "launcher.h"
#include "ui_launcher.h"
#include "filesystem.h"

#include <QProcess>

GIFAnimationDemoWidget::GIFAnimationDemoWidget( QWidget* parent ) :
    QWidget( parent, Qt::Window | Qt::FramelessWindowHint ),
    ui( new Ui::GIFAnimationDemoWidget )
{

    ui->setupUi(this);
    setWindowTitle("ВКО: Наследие v2 - обновление");
    ui->lbHint->setText("Получение списка файлов...");
    movie = new QMovie(":/second.gif");
    movie->setScaledSize(ui->lbMovie->size());
    movie->start();
    ui->lbMovie->setMovie(movie);

    data_downloader = new Downloader();
}

GIFAnimationDemoWidget::~GIFAnimationDemoWidget() {
    data_downloader->deleteLater();
    delete ui;
}

void GIFAnimationDemoWidget::startFilesDownload()
{
    ui->lbHint->setText("Начинаем проверку обновлений...");
    FileSystem::createFilePath(target_folder);
    data_downloader->setUrl(QUrl(base_url + "/filelist.txt"));
    data_downloader->targetBytearray = &downloaded_data;
    data_downloader->disconnect();
    connect(data_downloader, &Downloader::downloadFinished, this, &GIFAnimationDemoWidget::onFileListReady);
    data_downloader->start();
}

void GIFAnimationDemoWidget::resizeEvent(QResizeEvent *)
{
    qDebug() << "LOL" << ui->lbMovie->size();
    movie->setScaledSize(ui->lbMovie->size());
    ui->lbMovie->setMovie(movie);
}

void GIFAnimationDemoWidget::onFileListReady()
{
    const QString file_list(downloaded_data);
    qDebug() << "GOT FILE LIST: " << file_list;
    const QStringList files_info = file_list.split("\r\n");
    for (QString file_info: files_info) {
        const QStringList file_info_splitted = file_info.split("|");
        if (file_info_splitted.length() != 2) {
            continue;
        }
        const QString file_path = file_info_splitted[0];
        const QString file_hash = file_info_splitted[1];
        const QString target_file_path = target_folder + QString(file_path).replace("release/", "");

        if (!FileSystem::fileExists(target_file_path) || FileSystem::fileHash(target_file_path) != file_hash) {
            qDebug() << "Adding to queue file " << file_path;
            files_to_download.push_back(file_path);
        }
    }
    files_total = files_to_download.size();
    files_downloaded = 0;
    downloadNextEnqueuedFile();
}

void GIFAnimationDemoWidget::onDownloadNextEnqueuedFileFinished(Downloader *downloader)
{
    qDebug() << "Finished downloading:" << downloader->getUrl();
    files_downloaded += 1;
    downloader->targetFile->close();
    downloader->targetFile->deleteLater();
    downloadNextEnqueuedFile();
}

void GIFAnimationDemoWidget::onDownloadProgressChanged(Downloader *downloader, Downloader::Status status)
{
    QString download_status = "Загрузка файла " + QString::number(files_downloaded + 1) + " из " + QString::number(files_total) +
                              ": " + QString::number(status.percent, 'f', 1) + "% (" + Downloader::getSpeedFormatted(status.current_speed) + ")";

    ui->lbHint->setText(download_status);
}

void GIFAnimationDemoWidget::downloadNextEnqueuedFile()
{
    if (files_to_download.empty()) {
        ui->lbHint->setText("Загрузка завершена, запускаем Наследие...");
        startApplication();
        return;
    }
    const QString file_path = files_to_download.front();
    files_to_download.pop_front();
    const QString target_file_path = target_folder + QString(file_path).replace("release/", "");
    data_downloader->setUrl(QUrl(base_url + file_path));
    FileSystem::createFilePath(target_file_path);
    if (!(QFileInfo(target_file_path) == QFileInfo(QApplication::applicationFilePath()))) {
        QFile::remove(target_file_path);
    }

    data_downloader->targetFile = new QFile(target_file_path);
    data_downloader->targetFile->open(QIODevice::WriteOnly);
    if (!data_downloader->targetFile->isOpen() || !data_downloader->targetFile->isWritable()) {
        QFile::remove(target_file_path + ".forward");
        data_downloader->targetFile->setFileName(target_file_path + ".forward");
        data_downloader->targetFile->open(QIODevice::WriteOnly);
    }
    data_downloader->disconnect();
    connect(data_downloader, &Downloader::progressChanged, this, &GIFAnimationDemoWidget::onDownloadProgressChanged);
    connect(data_downloader, &Downloader::downloadFinished, this, &GIFAnimationDemoWidget::onDownloadNextEnqueuedFileFinished);
    data_downloader->start();
}

void GIFAnimationDemoWidget::startApplication()
{
    QProcess process;
    if (FileSystem::fileExists(target_folder + "LegacyForwardHelper.exe")) {
        process.setProgram(target_folder + "LegacyForwardHelper.exe");
    } else {
        process.setProgram(target_folder + "Legacy.exe");
    }
    process.setArguments({"-prelaunched"});
    process.startDetached();
    process.waitForFinished(-1);
    QMetaObject::invokeMethod(this, &GIFAnimationDemoWidget::close, Qt::QueuedConnection);
}

void GIFAnimationDemoWidget::startNewAnimation() {
    delete movie;
    movie = new QMovie(":/second.gif");
    ui->lbMovie->setMovie(movie);
    movie->setScaledSize(QSize(ui->lbMovie->width(), ui->lbMovie->height()));
    movie->start();
}
